<?php
/*

 $Source: /tmp/maintain/modules/class/host.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/



/**
 * host.php - defines the @see Host class 
 * @version $Id: host.php,v 1.99 2004/06/18 22:15:29 philips Exp $
 * @copyright Copyright (c) 2004 Oregon State University - Network Engineering
 * @filesource
 */

/**
 * Host - allows for updating modifying and viewing a host
 *
 * There is a little trick to the IP stuff on our hosts now. There
 * are two fields $ip and $dynamic_range. If the host is a dynamic host
 * then IP is 0 and dynamic_range is... the dynamic range. If it is 
 * a static host then dynamic_range is 0 and IP is the int value of 
 * the ip address. 
 * @package network_model
 */
class Host 
{

	//Var gathered from the host table
	/**
	 * Hosts id number - used in the database 
	 * @var		integer
	 * @access	public
	 */
	var $id;
	/**
	 * Hosts ip address - dynamic ips are set to 0 
	 * @var		integer
	 * @access	public
	 */
	var $ip;
	/**
	 * Hostname without domain ex. abe from abe.osuosl.org
	 * @var		string
	 * @access	public
	 */
	var $name;
	/**
	 * Hardware address as integer
	 * @var		integer
	 * @access	public
	 */
	var $ha;
	var $os;
	var $type;
	var $location;
	var $serial;
	var $other_id;
	/**
	 * Propogated by the rtr logs, @see bin/rtr-log-extract
	 * @var		mixed
	 * @access	public
	 */
	var $last_seen;
	var $enabled;
	/**
	* When was the host last updated by a user - unix time
	* @var		integer
	* @access	public
	*/
	var $last_update;
	/**
	* Last user to update the host (username) 
	* @var		string
	* @access	public
	*/
        var $who_update;
	/**
	 * 
	 * @var		enum (full,limited,blocked)
	 * @access	public
	 */
	var $bandwidth_rate;
	/**
	 * Date for deletion, stored as unix timestamp
	 * @var		integer
	 * @access	public
	 */
	var $expire; 
	/**
	 * Time to live for the host
	 * @var		integer
	 * @access	public
	 */
	var $ttl;

	/**
	 * The dyanmic range the host belongs to, as an object
	 * @var		DynamicRange
	 * @access	public
	 */
	var $dynamic_range;
	/**
	 * Domain of host, as object 
	 * @var		Domain
	 * @access	public
	 */
	var $domain;
	/**
	 * Workgroup of host, as object 
	 * @var		Workgroup
	 * @access	public
	 */
	var $workgroup;
	/**
	 * Zone of host, as object 
	 * @var		Zone
	 * @access	public
	 */
	var $zone;
	/**
	 * Full hardware address, : seperated 
	 * @var		string
	 * @access	public
	 */
	var $full_ha;
	/**
	 * host name + domain name (constructed)
	 * @var		string
	 * @access	public
	 */
	var $hostname;

	/**
	 * hosts history information 
	 * @var		ObjectHistory
	 * @access	public
	 */
	var $object_history;

	// we can populate this stuff from the database -- no sense in
	//   passing it in.

	/**
	*
	* Initializes a Host object if it exists
	*
	* The intialization is a fairly costly operation as the constructor
	* creates many objects that may or may not be used in the life of
	* the host object.  See the public variables for a list of all the
	* fields that are created.
	*
	* @return	bool	 False if host does not exist 
	* @access	public
	* @see		Host
	*/
	function Host ($host_id) { 
		global $conf;

		$this->id = $host_id;

		if ($host_id) {
			$info = $this->get_info();
		}

		if ( ! $info ) {
			return false;
		}

		//things found in the host table
		$this->name	      = $info->name;
		$this->ha	      = $info->ha;
		$this->os	      = $info->os;
		$this->type	      = $info->type;
		$this->location	      = $info->location;
		$this->serial	      = $info->serial;
		$this->other_id	      = $info->other_id;
		$this->enabled	      = $info->enabled;
		$this->bandwidth_rate = $info->bandwidth_rate;
		$this->purchase_date  = $info->purchase_date;
		$this->po_number      = $info->po_number;
		$this->warranty_date  = $info->warranty_date;
		$this->owning_unit    = $info->owning_unit;
		$this->user_id        = $info->user_id;
		$this->last_seen      = $info->last_seen;
		$this->last_update    = $info->last_update;
		$this->expire         = $info->expire;
		$this->ttl			  = $info->ttl;

		//Objects (requires more sql calls)
		$this->workgroup = new Workgroup($info->workgroup);
		$this->domain	 = new Domain($info->domain);
		$this->zone	 = new Zone($info->zone);

		// build the object_history object
		$this->object_history = new ObjectHistory('host', $this->id);

		//buid the full ha with :'s and everything!
		$built_ha = $info->ha{0} . $info->ha{1} . ":" . $info->ha{2} . $info->ha{3} . ":" . $info->ha{4} . $info->ha{5} . ":" . 
			    $info->ha{6} . $info->ha{7} . ":" . $info->ha{8} . $info->ha{9} . ":" . $info->ha{10} . $info->ha{11};
		$this->full_ha = $built_ha;
	
		// dynamic IP's are set to 0
		if ( $info->ip ) {
			$this->ip	= new IP($info->ip);
		}
		else {
			$this->dynamic_range = new Range($info->dynamic_range);
			$this->ip 	     = 0;
		}

		// do this last because we need Domain dummy
		$this->hostname = $this->name . "." . $this->domain->name;

	} // Host

	/**
	*
	* Returns the entire row for this host from the host table in the db
	*
	* @return	object	 mysql result set
	* @access	public
	* @see		http://php.oregonstate.edu/manual/en/function.mysql-fetch-object.php
	*/
	function get_info () { 

		$sql = "SELECT * FROM host WHERE id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		return mysql_fetch_object($db_results);
	} // get_info

	/**
	*
	* Returns the workgroups the host belongs could belong to
	*
	* This function discovers the workgroups a host CAN belong, not the 
	* workgroups it belongs to.  A VERY subtle difference.
	*
	* @return	Workgroup       Returns a linked array of workgroups
	* @access	public
	* @see		Workgroup
	* @param        integer         $zone_id Defaults to the hosts zone
	*                               and then the users zone
	*/
	function get_workgroups($zone_id =0) {
		global $conf;

		// Next we need to check the workgroup mappings
		if (!$zone_id) { $zone_id = $this->zone->id; } // Try hosts zone
		if (!$zone_id) { $zone_id = $conf['zone']->id; } // Try users zone
		$sql = "SELECT workgroup FROM zone_workgroup WHERE zone_workgroup.zone='$zone_id'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) {
			$workgroup = new Workgroup($r->workgroup);
			$results[$workgroup->name] = $workgroup;
		} // end while
		
		$default_group = $conf['ui']->get_workgroups("SELECT * FROM workgroup WHERE name LIKE '" . $conf['default_workgroup'] . "'");
		$results = array_merge($default_group,$results);
		
		return $results;

	} // get_workgroups

	// get_host_history
	function get_host_history($ip_int=0,$ha=0) {
		global $conf;

		// Check to see if we are using a real object
		// This is bassed of $this->ha
		if ($this->id) { 
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ha='$this->ha' ORDER BY date DESC";
		}
		//If they passed a ip_int base it on that
		elseif ($ip_int) {
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ip='$ip_int' ORDER BY date DESC";
		}
		//lastly go for the ha
		else {
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ha='$ha' ORDER BY date DESC";
		}

		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = $r;
		} //end while
	
	return $results;

	} // get_host_history

	/**
	*
	* Retrieve all dhcp options for this host
	*
	* Looks up the options that have been specified for this host (not
	* for the subnet or workgroup) and returns them as an array of Option
	* objects.  Searches the object_option table. 
	*
	* @return	Option	 an array of Options
	* @access	public
	* @see		Option
	*/
	function get_options() { 
		global $conf;

		$sql = "SELECT id FROM object_option WHERE object_id='$this->id' AND type='host'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_array($db_results)) {
			$results[] = new Option($r['id']);
		}

		return $results;

	} // get_options

	/**
	*
	* Get the IPs that this host has had in the past
	*
	* Searches the host_history table for the information by hardware
	* address.  The array is sorted in order by date.
	*
	* @return	array  an array of arrays, with the final fields being ->start, ->int, ->end 
	* @access	public
	*/
	function get_history_ips() { 

		$last_date = 0;

		$sql = "SELECT * FROM host_history WHERE ha='$this->ha' ORDER BY date ASC";
		$db_results = mysql_query($sql, dbh());

		while ($history = mysql_fetch_object($db_results)) {
		
			$data->start = $last_date;
			$data->int = $history->ip;
			$data->end = $history->date;
			$last_date = $history->date;
			$results[] = $data;

		} // while results

		return $results;

	} // end get_history_ips

	/**
	*
	* Checks to see if host has a history
	*
	* Uses the host_history table and the hardware address
	*
	* @return	mixed	 Returns a boolean false if there is no host
	*                        history and a mysql result set object if
	*                        there is
	* @access	public
	*/
	function has_host_history() {
		global $conf;

		$sql = "SELECT id FROM host_history WHERE ha='$this->ha' LIMIT 1";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		$r = @mysql_fetch_object($db_results);

		return $r;

	} //has_host_history

	/**
	*
	* Echos the date the host was last seen or N/A if it has no last_seen
	*
	* @access	public
	* @see		$last_seen
	*/
	function show_last_seen() { 
		if ( $this->last_seen ) {
        		echo date("F j, Y, g:i a", $this->last_seen);
		}
		else {
			echo "N/A";
		}
	}

	/**
	*
	* Echos the date the host will expire or N/A if it has no expire
	*
	* @access	public
	* @see		$expire
	*/
        function show_expire_date() {
		if ($this->expire) {
			echo date("F j, Y, g:i a", $this->expire);
		}
		else {
			echo "Never Expires";
		}
	} // show_expire_date


        /*
         * Simple Non-Checking Setters
         */


	function update_purchase_date($purchase_date) {
		$this->update_item('purchase_date', $purchase_date);
		$this->purchase_date = $purchase_date;
	}

	function update_zone($zone_id) {
		$this->update_item('zone',$zone_id);
		$this->zone = new Zone ($zone_id);
	}

	function update_po_number($po_number) {
		$this->update_item('po_number',$po_number);
		$this->po_number = $po_number;
	} 

	function update_warranty_date($warranty_date) {
		$this->update_item('warranty_date',$warranty_date);
		$this->warranty_date = $warranty_date;
	}

	function update_owning_unit($owning_unit) {
		$this->update_item('owning_unit',$owning_unit);
		$this->owning_unit = $owning_unit;
	}

	function update_user_id($user_id) {
		$this->update_item('user_id',$user_id);
		$this->user_id = $user_id;
	}

	function update_name ($newname) {
		$this->update_item('name', $newname);
		$this->name = $newname;
	}

	function update_enabled ($new_status) {
		$this->update_item('enabled',$new_status);
		$this->enabled = $new_status;
	}

	function update_bandwidth_rate ($new_rate) {
		$this->update_item('bandwidth_rate', $new_rate);
		$this->bandwidth_rate = $new_rate;
	} 

	function update_expire ($new_expire) { 
		$this->update_item('expire',$new_expire);
		$this->expire = $new_expire;
	}

	function update_ttl($new_ttl) {
		$this->update_item('ttl',$new_ttl);
		$this->ttl = $new_ttl;
	}

	function update_workgroup ($new_workgroup) {
		$this->update_item('workgroup',$new_workgroup); 
		$this->workgroup = $new_workgroup;
	}

	function update_domain ($new_domain) {
		$this->update_item('domain', $new_domain);
		$this->domain = new Domain($new_domain);
	}

	function update_ha ( $new_ha ) {
		$this->update_item('ha', $new_ha);
		$this->ha = $new_ha;
	}

	function update_type ( $new_type ) {
		$this->update_item('type', $new_type);
		$this->type = $new_type;
	}

	function update_os ( $new_os ) {
		$this->update_item('os', $new_os);
		$this->os = $new_os;
	}

	function update_location ( $new_location) {
		$this->update_item('location', $new_location);
		$this->location = $new_location;
	}

	function update_serial ( $new_serial ) {
		$this->update_item('serial', $new_serial);
		$this->serial = $new_serial;
	}

	function update_other_id ( $new_other_id ) {
		$this->update_item('other_id', $new_other_id);
		$this->other_id = $new_other_id;
	}


	/**
	*
	* Updates the dynamic_range or ip address
	*
	* This single method can be used to update the ip address of a host OR
	* tell the host to use a dynamic range.
	*
	* @access	public
	* @see		update_item
	* @param        integer         $new_ip  The new ip of the host, if
	*                               this is given the range can be ignored.
	*                               NOTE: IP given in integer form.
	* @param        integer         $new_range    The new range ID of the 
	*                               host.  $new_ip must be 0 for the range
	*                               to be used.
	*/
	function update_ip ($new_ip, $new_range=0) {
		global $conf; 

		if ($new_ip) {
			$this->update_item('ip', $new_ip);
			$this->ip = $new_ip;	
			
			//just incase let's kill the dynamic_range entry
			$this->update_item('dynamic_range',"0");	
			$this->dynamic_range = "0";		
		}
		elseif ($new_range) {
			$this->update_item('dynamic_range',$new_range);
			$this->dynamic_range = $new_range;

			//just incase let's kill the ip entry
			$this->update_item('ip',"0");
			$this->ip="0";
		}

	} //update_ip

	/**
	*
	* Updates a field (in the database) for this host
	*
	* This command executes UPDATE host SET $type='$new_item' WHERE id='$this->id'.
	*
	* @access	public
	* @param	string	$type		the field to update in the 
	*					table	
	* @param 	string	$new_item	text to place in field 
	*					$type.
	*/
	function update_item ($type, $new_item) {
		global $user;

		// do a last min check to make sure they have access
		if ($user->has_access($this->zone->id)) {

			$new_item = sql_escape($new_item);

			$sql = "UPDATE host SET $type='$new_item' WHERE id='$this->id'";
			$db_result = mysql_query($sql, dbh());
		}

	} //update_item

	/**
	*
	* Check to see if host is active
	*
	* Uses $conf['update_time'] and compares it to the current time.  If
	* The host has been seen since we last update then it is assumed 
	* active.
	*
	* @return	boolean	true if the host has been seen, false otherwise
	* @access	public
	*/
	function is_active() { 
		global $conf;

		$history = $this->object_history->get_history();

		if ($history->date + $conf['update_time'] <= time()) {
			return true;
		}

		return false;

	} //is_active

	/**
	*
	* Deletes a host and all information connected to it
	*
	* Deletes the host from the following tables:
	* - host
	* - object_option
	* - object_history
	*
	* Although this function does not return anything it returns things
	* behind the scenes by using ui->add_delete_history
	* @return	null
	* @access	public
	* @see 		UI::add_delete_history()
	*/
	function delete() { 
		global $conf,$user;
		
		// Delete the host!
		$sql = "DELETE FROM host WHERE id='$this->id'";
		$db_result = mysql_query($sql, dbh());

		// Delete any options..
		$sql = "DELETE FROM object_option WHERE type='host' AND object_id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		// Delete any object_history
		$sql = "DELETE FROM object_history WHERE type='host' AND object_id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		// Add the deletion record
		$text = "Deleted Host " . $this->hostname . "(" . $this->full_ha . ")";
		if ($this->ip->ip_int) {
			$text .= " IP:" . $this->ip->ip_addr;
		}
		else {
			$text .= " Dynamic Address:" . $this->dynamic_range->start->ip_addr;
		}
		$conf['ui']->add_delete_history($this->zone->id, $text, $user->username);

	} //delete

	/**
	*
	* Gets (and creates) an array of hosts based on the parameters 
	*
	*
	* @deprecated 	This should be deprecated soon and put into a seperate
	*		wrapper class
	* @return	boolean	 Description
	* @access	public
	* @param 	$select_type	Selects the type of search.
	*				- Defined Range - range ID @see Range 
	*				- Custom Range - by start and end ranges
	*				- Domain - via domain id @see Domain
	*				- Hostname - via human readable hostname
	*				- Other - includes all available host 
	*				  fields and a $value
	* @param	$range_id	For "Defined Range" @see Range
	* @param	$start_range	For "Custom Range" - start of IP range search
	* @param	$end_range	For "Custom Range" - end of IP range search
	* @param	$domain_id	For "Domain" - @see Domain
	* @param 	$hostname	For "Hostname" 
	* @param	$select_field	For custom search
	* @param	$value		For custom search - the value of the 
	*				field
	*/
	function bulk_get($select_type,$range_id=0,$start_range=0,$end_range=0,
			  $domain_id=0,$hostname=0,$select_field=0,$value=0) { 
		global $conf;

                //ok we've got to get the hosts for viewing
                //check to see what kind of select they did and grab the
                //correct hosts

                //zone -> range grab
                if ($select_type == "defined_range") {
                        $range = new Range($range_id);
                        $hosts = $range->get_range_hosts();
                }
                //custom range grab
                elseif ($select_type == "custom_range") {
                //check to make sure range was entered correctly
                if (! Net_IPv4::validateIP($start_range) ) {
                // if (! preg_match("/^\d+\.\d+\.\d+\.\d+$/", $start_range) ) {
                        $conf['error']->add_error('start_range', "Invalid Start Range");
                }
                if (! Net_IPv4::validateIP($end_range) ) {
                        $conf['error']->add_error('end_range', "Invalid End Range");
                }
                $start_int = ip2long($start_range);
                $end_int = ip2long($end_range);
                if ($start_int >= $end_int) {
                        $conf['error']->add_error('end_range', "End range is before start range");
                }
                if (!$conf['error']->error_state) {
                        $zone_id = $conf['zone']->id;
                        $hosts = $conf['ui']->get_hosts("SELECT id FROM host WHERE ip<='$end_int' AND ip>='$start_int' AND zone='$zone_id' ORDER BY ip DESC");

                } //check for error state

                } //custom range grab

                //zone -> domain grab
                elseif ($select_type == "domain") {
                        $domain = new Domain($domain_id);
                        $hosts = $domain->get_hosts();
                }
                //fuzzy host grab
                elseif ($select_type == "hostname") {
                        if (substr($hostname,strlen($hostname)-1,1) != "%") { $hostname = $hostname ."%"; }
                        if (substr($hostname,0,1) != "%") { $hostname = "%" . $hostname; }

                        $zone_id = $conf['zone']->id;
                        $hosts = $conf['ui']->get_hosts("SELECT * FROM host WHERE name LIKE '$hostname' AND zone='$zone_id'");
                }

		elseif ($select_type == 'other') {
			$zone_id = $conf['zone']->id;
			$value = "%" . $value . "%";
			$sql = "SELECT id FROM host WHERE $select_field LIKE '$value' AND zone='$zone_id'";
			$hosts = $conf['ui']->get_hosts($sql);
		}
		
		return $hosts;

	} // bulk_get

	/**
	*
	* Checks to make sure a workgroup can be added to the specified zone.
	*
	* Note: If the user is an Admin no checking is done
	*
	* @return	bool	True if workgroup is allowed in the zone or the user is an 
	*					admin, False otherwise
	* @access	public
	* @param	$workgroup	@see Wourkgroup object
	* @param	$zone_id	A zone id, or nothing for this hosts zone
	*/
	function check_workgroup($workgroup, $zone_id=0) { 
	
		if ($user->is_admin) { return true; }
		if (!$zone_id) { $zone_id = $this->zone->id; }
	
		return $workgroup->has_zone($zone_id);

	} // check_workgroup


	/*!
		@function check_hostname
		@discussion the hostname (not fully-qualified) and a Domain object
	*/
	/**
	*
	* Checks to ensure that the hostname follows all of the proper rules 
	*
	* The following checks are done:
	* - $name is not blank
	* - $name is not an IP (because of dynamic hosts)
	* - $name is not a duplicate hostname for this domain (checks case)
	* - $name is not the same name as a forward pointer
	* - $name is not a CName
	*
	* Side effects: On error puts out information using error object
	*
	* @return	bool		True if the hostname is ok, false otherwise
	* @access	public
	* @param	$name		The host name (not fully qualified) 	
	* @param	$domain 	@see Domain a domain object
	* @param	$host_id 	ID to ignore, if we are already working on that id	
	*/
	function check_hostname($name, $domain, $host_id = 0) {
		global $conf;

		if (strlen($name) < 1) { 
			$conf['error']->add_error('name',"No name entered. Please enter a name for this host.");
		}

		// because we format our dynamic hosts 128-193-188-118.yoursite.com we can't let people
		// create hosts like that normaly
		if ( Net_IPv4::validateIP($name) ) {
			$conf['error']->add_error('name', "$name not allowed, it would overlap with auto-generated dynamic hosts");
		}

		if (!preg_match("/^[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]+/", $name) ) {
			$conf['error']->add_error('name', "$name.$domain->name is an invalid hostname.");	
			return false;	
		}
		
		// we have to use the host_id to determine if they working on that host currently
		//   this is used during the update of a single host
		// we do a like here in so we get the case-insensitive match
		$sql = "SELECT * FROM host WHERE name LIKE '$name' AND domain='$domain->id'";
		if ($host_id) { $sql .= " AND id!='$host_id'"; }
		$db_result = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_object($db_result);
		if ( $host_info ) {
			if ( $host_info->id != $host_id ) {
				$link = "<a href=\"" . $conf['web_path'] . "/host.php?action=show_host&host_id=" . $host_info->id . "\">[ Show Host ]</a>";
				$conf['error']->add_error('name', "$name.$domain->name is already in use by a Host. " . $link);	
				}
		} // if host_info


		// now we need to make sure there isn't a forward pointer with this name.. 
		$sql = "SELECT id FROM pointer WHERE hostname LIKE '" . $name . $domain->name . "'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_row($db_results);
	
		if ($host_info) {
			$link = "<a href=\"" . $conf['web_path'] . "/pointer.php?action=show_pointer&pointer_id=" . $host_info[0] . "\">[ Show Pointer ]</a>";
			$conf['error']->add_error('name', "$name.$domain->name is already in use by a Pointer. " . $link);
		}

		// now we need to check to make sure there isn't a Cname with this name.
		$sql = "SELECT id FROM zone_cname WHERE name LIKE '$name' AND domain='$domain->id'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_row($db_results);
		
		if ($host_info) {
			$link = "<a href=\"" . $conf['web_path'] . "/cname.php?action=show_cname&cname_id=" . $host_info[0] . "\">[ Show Cname ]</a>";
			$conf['error']->add_error('name', "$name.$domain->name is already in use by a CNAME. " . $link);
		}

		if ($conf['error']->error_state) {
			return false;
		}

		return true;
		

	} //check_hostname

	/**
	*
	* Check to ensure the hardware address is correct and not a duplicate
	*
	* Uses Zone OR Subnet information to check that the HA is not a duplicate 
	* Side effects:
	* - Adds an error using the error object, therefore it returns nothing
	*
	* @return	null
	* @access	public
	* @param	$ha			Hardware address
	* @param	$host_id	Hosts id, if 0 uses $this	
	* @param	$range_id	
	*/
	function check_hardware_address($ha, $host_id = 0, $range_id,$subnet_id=0) {
		global $conf;

		//for range testing
		$range = new Range($range_id);

		// is it 12 chr's long
		if ( strlen($ha) != 12 ) {
			$conf['error']->add_error('ha', "Hardware address must be exactly 12 characters long.");	
		}
		
		// this is a special case for people who don't want to
		//   be in DHCP ... some ranges don't want to be there.
		// This is sort of kludgy but given the extremely special cases
		//  that it is used in it will have to remain, for now.
		// This option can now be set by Zone admins on a per zone basis so that
		//  we can control some of the rampant abuse of this feature.
		if (($ha == '000000000000') && (!$this->zone->get_allow_blank_ha())) {
			$conf['error']->add_error('ha',
				"Hardware address cannot be all zeros (which allows you to bypass DHCP). " .
				"Your admin has turned this option off.");	
		}

		// does it have valid chrs
		if (preg_match("/[^0-9a-fA-F]+/", $ha) ) {
			$conf['error']->add_error('ha',
				"Hardware address can only contain hexidecimal numbers (0-9 and A-F).");	
		}
	
		//check for duplicate ha in same zone
		if ($range->type == 'dynamic' OR $range->type == 'ddns') {
			$sql = "SELECT host.id FROM host WHERE ha='$ha' AND dynamic_range='$range_id'";
		}
		else {
			$sql = "SELECT host.id FROM host,ranges WHERE ranges.subnet='".$subnet_id."' AND host.ip<=ranges.end AND host.ip>=ranges.start  AND host.ha='". $ha."' ";
		}
		
		// this is for the case when we are actually updating a host
		if ( $host_id ) {
			$sql .= "AND host.id!='$host_id'";
		}

		$db_result = mysql_query($sql, dbh());
		$host_info = mysql_fetch_object($db_result);
		

		// Ensure their are no duplicates but allow all zeros
		if ( $host_info && ($ha != '000000000000')) {
			$conf['error']->add_error('ha', "The hardware address $ha is already in use.");	
		}
		
	} //check_hostname

	/**
	*
	* Check to make sure the IP range has free addresses.
	*
	* Sideffects:
	* - Uses the error object to put up a message about the range.
	*
	* @access	public
	* @see		Range::get_free_address_count()
	* @deprecated 		Use Range::get_free_address_count() instead
	*/
	function check_ip_range($range_id) { 
		global $conf;

		$range = new Range($range_id);
		if (! ( $range->get_free_address_count() ) ) {
			$conf['error']->add_error('ip_range', "There are no free IP addresses in this range.");
		}

	} //check_ip_range


	// For $this host and this zone return the
	//   next host object for this zone.  If there isn't
	//   another host then return nothing.
	/**
	* 
	* Return the next host id for this zone. 
	*
	* 
	*
	* This is the long description for the Class
	*
	* @return	mixed	The ID of the next host in the zone.
	*					If there isn't another host return nothing. 
	* @access	public
	* @deprecated		This function will be moved to Zone	
	*/
	function get_next_host_by_name () {

		$zone_id = $this->zone->id;
		$sql = "SELECT id FROM host WHERE zone=$zone_id ORDER BY name";
		$db_result = mysql_query($sql, dbh());

		while ( $host = mysql_fetch_array($db_result) ) {
			if ( $host[0] == $this->id ) {
				$host = mysql_fetch_array($db_result);
				return $host[0];
			}
		}

		return;
	} // get_next_host_by_name


	// For $this host and this zone return the
	//   next host object for this zone by IP address.

	/**
	*
	* Return the next IP in the zone
	*
	* @return	mixed	 Description
	* @access	public
	* @see		??
	* @deprecated		This function will be moved to Zone	
	*/
	function get_next_host_by_ip () {

		//if this is a static host then simply get the next possible ip address (get's next in line...)
		if ($this->ip->ip_int) {
			// increment the IP addr and make new object
			$ip_int = $this->ip->ip_int;
			$ip_int++;
	
			$zone_id = $this->zone->id;
			//All we have to do is order by ip on the host table
			//let's keep it in the same zone though
			$sql = "SELECT * FROM host WHERE ip>='$ip_int' AND zone='$zone_id' ORDER BY ip LIMIT 1";
		}
		//if this is a dynamic host then get the next dynamic host by id
		elseif ($this->dynamic_range->id) {
			//we are dealing with a dynamic address.... 
			$sql = "SELECT * FROM host WHERE dynamic_range='$this->dynamic_range->id' AND zone='$this->zone->id' AND id!='$this->id' ORDER BY id LIMIT 1";
		}

		$db_results = mysql_query($sql, dbh());
		$results = mysql_fetch_row($db_results);

		return $results[0];
	} //get_next_host_by_ip


	/**
	*
	* Show the workgroup or a pulldown box with different workgroups
	*
	* It does one of two things, if the host only belongs to only one group
	* then a hidden field "workgroup_id" is created.  (Does this case happen?)
	* Else a dropdown menu is created where the user can select the workgroup
	*
	* @access	public
	*/
	function show_workgroup_select() {
		global $conf;

		$workgroups = $this->get_workgroups();
		// if its > 1 then show the show_workgroup_select tool
		if ( count($workgroups) > 1 ) {
			$conf['ui']->show_workgroup_select($workgroups, $this->workgroup->id);
		}
		// else just put a read-only version
		else {
			$workgroup = array_pop($workgroups);
			$workgroup_name = $workgroup->name;
			print "<input type=\"hidden\" name=\"workgroup_id\" value=\"" . $workgroup->id . "\" />$workgroup_name";
		}

	} //show_workgroup_select

	/**
	*
	* Sends a notification to all people who "own" this host
	*
	* Uses the UI::notify_admins() function to actually do the sending 
	*
	* @return	string	 Description
	* @access	public
	* @see		UI::notify_admins()
	*/
	function send_notify($zones=0,$comment=0) {
		global $conf,$user;

		if (!$zones) { $zones = $this->zone; }

		if (!$conf['send_notifications']) { return false; }	
	
		$text = $this->hostname . " updated by an Administrator\n\n" .
		"Extra Comment: $comment\n" .
			"HW:$this->ha\nDomain:" . $this->domain->name . "\nOS:$this->os\nType:$this->type\n" . 
			"Location:$this->location\nSerial:$this->serial\nOther_id:$this->other_id\nStatus:$this->enabled\n" .
			"Purchase Date:$this->purchase_date\nPO Number:$this->po_number\nWarranty Date:$this->warranty_date\n" .
			"Owning Unit:$this->owning_unit\nUser ID:$this->user_id\nTTL:$this->ttl\n\nUpdated by $user->username";
	    
		$conf['ui']->notify_admins($text,"Host Updated",$zones);

    } // send_notify


    // Accessor functions -- things only used by this class or for
    //    debugging purposes
    function _show () {
    	print("Host: name -> $this->name, tc -> $this->tc, ");
	$this->ip->_show();
	$this->domain->_show();
	$this->zone->_show();
	print("end IP<br>\n");
    }

    function _debug () {
	if ( $this->debug ) {
		$this->debug = false;
	}
	else {
	    $this->debug = true;
        }
    }

} // class Host

?>
