<?php
/*

 $Id: update.php,v 1.12 2004/06/29 17:43:58 philips Exp $
 $Source: /tmp/maintain/modules/class/update.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header Update Class
	@discussion this class handles updating from one version of 
	maintain to the next. Versions are a 6 digit number
	220000
	^
	Major Revision
	
	220000
	 ^
	 Minor Revision

	The last 4 digits are a build number...
	If Minor can't go over 9 Major can go as high as we want
*/

class Update {

	var $key;
	var $value;
	var $versions; // array containing version information

	/*!
		@function Update
		@discussion Constructor, pulls out information about
			the desired key
	*/
	function Update ( $key=0 ) {

		if ($key) {
			$info = $this->get_info();
			$this->key = $key;
			$this->value = $info->value;
			$this->versions = $this->populate_version();
		}

	} // constructor

	/*!
		@function get_info
		@discussion gets the information for the zone
	*/
	function get_info() {
		global $conf;

		$sql = "SELECT * FROM update_info WHERE key='$this->key'";
		$db_results = mysql_query($sql, dbh());

		return mysql_fetch_object($db_results);		

	} //get_info

	/*!
		@function get_version
		@discussion this checks to see what version you are currently running
			because we may not have the update_info table we have to check 
			for it's existance first. 
	*/
	function get_version() {


		/* Make sure that update_info exits */
		$sql = "SHOW TABLES LIKE 'update_info'";
		$db_results = mysql_query($sql, dbh());
		// If no table
		if (!mysql_num_rows($db_results)) {
			
			$version = '220000';		
			
		} // if table isn't found

		else {
			// If we've found the update_info table, let's get the version from it
			$sql = "SELECT * FROM update_info WHERE `key`='db_version'";
			$db_results = mysql_query($sql, dbh());
			$results = mysql_fetch_object($db_results);
			$version = $results->value;
		} 

		return $version;

	} // get_version

	/*!
		@function format_version
		@discussion make the version number pretty
	*/
	function format_version($data) {

		$new_version = substr($data,0,strlen($data) - 5) . "." . substr($data,strlen($data)-5,1) . " Build:" . 
				substr($data,strlen($data)-4,strlen($data));

		return $new_version;

	} // format_version

	/*!
		@function need_update
		@discussion checks to see if we need to update 
			maintain at all
	*/
	function need_update() {

		$current_version = $this->get_version();

		if (!is_array($this->versions)) {
			$this->versions = $this->populate_version();
		}

		/* 
		   Go through the versions we have and see if
		   we need to apply any updates
		*/
		foreach ($this->versions as $update) {
			if ($update['version'] > $current_version) {
				return true;
			}

		} // end foreach version

		return false;

	} // need_update


	/*!
		@function populate_version
		@discussion just sets an array the current differences
			that require an update
	*/
	function populate_version() {

		/* Define the array */
		$version = array();
		
		/* Version 2.4 Build 0001 */
		$update_string = "<li>Add update_info table to the database</li>" . 
				 "<li>Add custom ENUM('true','false'), value VARCHR(255) to dhcp options</li>" . 
				 "<li>Add class enum to object_option type and scope" .
				 "<li>Add dhcp_class table to the database</li>" .
				 "<li>Add some missing Indexes</li>" .
				 "<li>Changed status=true,false and enabled=true,false into enabled= smallint(1) allowing boolean comparisons</li>" .
				 "<li>Added parent field to ranges for use with grouped ranges</li>" .
				 "<li>Added email field to user</li>";
		$version[] = array('version' => '240001', 'description' => $update_string);
		return $version;

	} // populate_version

	/*!
		@function display_update
		@discussion This displays a list of the needed
			updates to the database. This will actually
			echo out the list...
	*/
	function display_update() {

		$current_version = $this->get_version();
		if (!is_array($this->versions)) {
			$this->versions = $this->populate_version();
		} 

		echo "<ul>\n";

		foreach ($this->versions as $version) {
		
			if ($version['version'] > $current_version) {
				$updated = true;
				echo "<b>Version: " . $this->format_version($version['version']) . "</b><br />";
				echo $version['description']; 
			} // if newer

		} // foreach versions

		echo "</ul>\n";

		if (!$updated) { echo "<p align=\"center\">No Updates Needed</p>"; }
	} // display_update

	/*!
		@function run_update
		@discussion This function actually updates the db.
			it goes through versions and finds the ones 
			that need to be run. Checking to make sure
			the function exists first.
	*/
	function run_update() {
	
		/* Nuke All Active sessions  */
		$sql = "DELETE * FROM session";
		$db_results = mysql_query($sql, dbh());

	
		$methods = array();
		
		$current_version = $this->get_version();
		
		$methods = get_class_methods('Update');
		
		if (!is_array($this->versions)) { 
			$this->versions = $this->populate_version();
		}

		foreach ($this->versions as $version) { 


			// If it's newer than our current version
			// let's see if a function exists and run the 
			// bugger
			if ($version['version'] > $current_version) { 
				$update_function = "update_" . $version['version'];
				if (in_array($update_function,$methods)) {
					$this->{$update_function}();
				}

			} 
		
		} // end foreach version

	} // run_update

	/*!
		@function set_version
		@discussion sets a new version takes
			a key and value
	*/
	function set_version($key,$value) {

		$sql = "UPDATE update_info SET value='$value' WHERE `key`='$key'";
		$db_results = mysql_query($sql, dbh());		

	} //set_version

	/*!
		@function update_240001
		@discussion Migration function for 2.4 Build 0001
	*/
	function update_240001() {

		/* Create the Update Info Table */
		$sql = "CREATE TABLE `update_info` (`key` VARCHAR( 128 ) NOT NULL ,`value` VARCHAR( 255 ) NOT NULL ,INDEX ( `key` ) )";
		$db_results = mysql_query($sql, dbh());

		/* Insert the db_version var */
		$sql = "INSERT INTO update_info (`key`,`value`) VALUES ('db_version','220000')";
		$db_results = mysql_query($sql, dbh());

		/* Modified the dhcp_options table and add a index */
		$sql = "ALTER TABLE `dhcp_options` ADD `custom` SMALLINT( 1 ) DEFAULT '0' NOT NULL , ADD `value` VARCHAR( 255 ) NOT NULL " . 
			", ADD `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL , ADD `space` VARCHAR( 255 ) NOT NULL";
		$db_results = mysql_query($sql, dbh());

		$sql = "ALTER TABLE `dhcp_options` ADD INDEX ( `custom` )";
		$db_results = mysql_query($sql, dbh());

		/* Created dhcp_class table */
		$sql = "CREATE TABLE `dhcp_class` ( `id` INT( 11 ) UNSIGNED NOT NULL AUTO_INCREMENT , `name` VARCHAR( 255 ) NOT NULL , " .
				"`space` VARCHAR( 255 ) NOT NULL , `subclass` SMALLINT( 1 ) DEFAULT '0' NOT NULL ," . 
				" `parent_class` VARCHAR( 255 ) NOT NULL , PRIMARY KEY ( `id` ) )";
		$db_results = mysql_query($sql, dbh());

		/* Add Class to object_option */
		$sql = "ALTER TABLE `object_option` CHANGE `type` `type` ENUM( 'host', 'subnet', 'workgroup', 'global', 'class' ) DEFAULT 'host' NOT NULL ";
		$db_results = mysql_query($sql, dbh());

		$sq = "ALTER TABLE `object_option` CHANGE `scope` `scope` ENUM( 'host', 'subnet', 'workgroup', 'global', 'class' ) DEFAULT 'host' NOT NULL ";
		$db_results = mysql_query($sql, dbh());

		/* Add a Index for type on ranges (we select this during builds) */
		$sql = "ALTER TABLE `ranges` ADD INDEX ( `type` )";
		$db_results = mysql_query($sql, dbh());

		/* Add a index on zone in zone_domain */
		$sql = "ALTER TABLE `zone_domain` ADD INDEX ( `zone` )";
		$db_results = mysql_query($sql, dbh());

		/* Add a new field to range (PARENT) */
		$sql = "ALTER TABLE `ranges` ADD `parent` INT(11) UNSIGNED NOT NULL";
		$db_results = mysql_query($sql, dbh());

		$data = array();

		/* Round up all disabled ranges so after we've made the change me can make sure
		   they are still disabled
		*/
		$sql = "SELECT * FROM `ranges` WHERE enabled='false'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$data[] = $r->id;
		}

		/* Make Enabled boolean rather than string, makes comparison easier */
		$sql = "ALTER TABLE `ranges` CHANGE `enabled` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Now go back and set all the disabled ranges again... */
		foreach ($data as $value) { 
			$sql = "UPDATE `ranges` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();

		/* Round up all of the disabled hosts... */
		$sql = "SELECT * FROM `host` WHERE status='disabled'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) {
			$data[] = $r->id; 
		}

		/* Alter the table */
		$sql = "ALTER TABLE `host` CHANGE `status` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Now go back and re-populate */
		foreach ($data as $value) { 
			$sql = "UPDATE `host` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();

		/* Round up all of the disable domains */
		$sql = "SELECT * FROM `domain` WHERE status='disabled'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$data[] = $r->id; 
		}
	
		/* Alter the table */
		$sql = "ALTER TABLE `domain` CHANGE `status` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Let's set all those mofos back */
		foreach ($data as $value) { 
			$sql = "UPDATE `domain` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();

		/* Round up all of the disabled ip_blocks */
		$sql = "SELECT * FROM `ip_blocks` WHERE enabled='false'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$data[] = $r->id;
		}
		
		/* Alter the table */
		$sql = "ALTER TABLE `ip_blocks` CHANGE `enabled` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());
		
		/* set all the mofos back */
		foreach ($data as $value) { 
			$sql = "UPDATE `ip_blocks` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();


		/* Round up all of the disabled pointers */
		$sql = "SELECT * FROM `pointer` WHERE status='disabled'";
		$db_results = mysql_query($sql, dbh());
		
		while ($r = mysql_fetch_object($db_results)) {
			$data[] = $r->id;
		}

		/* Alter the table */
		$sql = "ALTER TABLE `pointer` CHANGE `status` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Put the mofo's back */
		foreach ($data as $value) { 
			$sql = "UPDATE `pointer` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();

		/* Add enabled to CNAMES */
		$sql = "ALTER TABLE `zone_cname` ADD `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Add enabled to MX Records */
		$sql = "ALTER TABLE `zone_mx` ADD `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Round up all of the disabled pointers */
		$sql = "SELECT * FROM `zone_range` WHERE enabled='false'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$data[] = $r->id;
		}

		/* Alter the table */
		$sql = "ALTER TABLE `zone_range` CHANGE `enabled` `enabled` SMALLINT( 1 ) DEFAULT '1' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Put the mofo's back */
		foreach ($data as $value) { 
			$sql = "UPDATE `zone_range` SET enabled='0' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();

		/* Round up all the default workgroups */
		$sql = "SELECT * FROM `zone_workgroup` WHERE is_default='true'";
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$data[] = $r->id;
		}

		/* Alter the table */
		$sql = "ALTER TABLE `zone_workgroup` CHANGE `is_default` `is_default` SMALLINT( 1 ) DEFAULT '0' NOT NULL";
		$db_results = mysql_query($sql, dbh());

		/* Put the mofo's back */
		foreach ($data as $value) { 
			$sql = "UPDATE `zone_workgroup` SET is_default='1' WHERE id='$value'";
			$db_results = mysql_query($sql, dbh());
		}
		$data = array();
		
		/* Add email field to user*/
		$sql =	"ALTER TABLE `user` ADD `email` VARCHAR( 255 ) default NULL";

		$db_results = mysql_query($sql, dbh());

		/* Add allow_all_hosts field to user*/
		$sql =	"ALTER TABLE `ranges` ADD `allow_all_hosts` SMALLINT( 1 ) default 0";

		$db_results = mysql_query($sql, dbh());

		/* Add allow_blank_ha to zone */
		$sql = "ALTER TABLE `zone` ADD `allow_blank_ha` SMALLINT( 1 ) DEFAULT '0' NOT NULL";
		
		$db_results = mysql_query($sql, dbh());

		$this->set_version('db_version','240001');

		return true;

	} // update_240001

	
} // end update class

?>
